//****************************************************
//***** AITENDO 128x64 OLED Graphic with GrayScale ***
//*************************** By Minarai Shokunin ****
//*************************** 2013/09/21          ****
//****************************************************

#include "p24FJ64GA002.h"
#include "oled.h"
#include "font8x8.h"

extern const unsigned char font8x8[];
extern unsigned int GSVirtualScreen[8][128];
extern unsigned int ch8x8[];

// ************ prototypes ***************************
void GR_FILL_GS(unsigned char);
void GR_PSET_GS(int,int,unsigned char);
void GR_PUTCH_GS(int,int,unsigned char,unsigned char);
void GR_LINE_GS(int,int,int,int,unsigned char);
void GR_PUTCH_GS(int,int,unsigned char, unsigned char);
void GR_PUTSTR_GS(int,int,unsigned char *, unsigned char);
void GR_PUT8x8(int,int,int);

// ************ Values ***************************
const unsigned int bits[] = {1,2,4,8,16,32,64,128};
const unsigned int mask[] = {0xfefe,0xfdfd,0xfbfb,0xf7f7,0xefef,0xdfdf,0xbfbf,0x7f7f};

// ************ 8x8 gray scale put sub routine ***************************

void GR_PUT8x8(int x,int y,int ch) {
	int y0,y1,i,p0;
	unsigned int c0,c1;
	
	if ((x<0)||(x>120)||(y<0)||(y>56)) {return ;}	
	
	y0=y/8; y1=y & 0x0007;
	p0 = ch * 8;

	for (i=0;i<8;i++) {
		c0 = ch8x8[p0+i]<<y1; 		c1 = ch8x8[p0+i]>>(8-y1);
		GSVirtualScreen[y0][x+i] |= c0;
	//	GSVirtualScreen[y0+1][x+i] |= c1;
	}
		
}
//********** Fill data *************
void GR_FILL_GS(unsigned char dt) {
	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<128;j++) {
			GSVirtualScreen[i][j] = dt;
		}
	}
}

//********** Put string *************
void GR_PUTSTR_GS(int x,int y,unsigned char *dt, unsigned char col) {
	unsigned char *p0;
	int x0;	
	
	p0 = dt;
	x0 = x;

	while (*p0 != 0x00) {
		GR_PUTCH_GS(x0,y,*p0++,col);
		x0 += 8;
	}	
}

//********** Put Number *************
void GR_PUTNUM_GS(int x,int y,int v, unsigned char col) {
	unsigned char v0;
	int i,x0,vv;
	
	x0 = x+3*8;
	vv=v;

	for (i=0;i<4;i++) {
		GR_PUTCH_GS(x0,y,0x30+(vv%10),col);
		x0 -= 8;
		vv /=10;
	}
}



//********** Put character *************

void GR_PUTCH_GS(int x,int y,unsigned char dt, unsigned char col) {

	int y0,y1,i,p0;
	unsigned int c0,c1;
	
	if ((x<0)||(x>120)||(y<0)||(y>56)) {return ;}	
	
	y0=y/8; y1=y & 0x0007;
	p0 = (dt - 0x20) * 8;

	for (i=0;i<8;i++) {
		if ((col & 0x01)== 0x01) {
			GSVirtualScreen[y0][x+i] |= font8x8[p0+i];
		}
		if ((col & 0x02)==0x02) {
			GSVirtualScreen[y0][x+i] |= font8x8[p0+i] << 8 ;
		}
	}
}


//********** Pset *************
void GR_PSET_GS(int x,int y,unsigned char dt) {
	int y0,y1;
	unsigned int c0,c1,cm;

	if ((x<0)||(x>127)||(y<0)||(y>63)) {return ;}	 

	y0=y/8; y1=y & 0x0007;
	c0 = GSVirtualScreen[y0][x];
	c1 = bits[y1];
	cm = mask[y1];
	if (dt ==0 ) {c1 =0;}
	if (dt ==2 ) {c1 <<= 8;}
	if (dt ==3 ) {c1 += (c1 << 8);}
	GSVirtualScreen[y0][x] = (c0 & cm) | c1;	
}

//********** Line *************

void GR_LINE_GS(int x0,int y0,int x1,int y1,unsigned char col)
{
  int i,dx,dy,e,sx,sy,xx0,yy0;

  xx0=x0;
  yy0=y0;

  if (x1 > x0) {
		dx = x1-x0;
		sx = 1;
  }else{
		dx = x0-x1;
		sx = -1;
  }

  if (y1 > y0) {
		dy = y1-y0;
		sy = 1;
  }else{
		dy = y0-y1;
		sy = -1;
  }

  if ( dx > dy ) {
    e = -dx;
    for ( i = 0 ; i <= dx ; i++ ) {
      GR_PSET_GS(xx0,yy0, col );
      xx0 += sx;
      e += 2 * dy;
      if ( e >= 0 ) {
        yy0 += sy;
        e -= 2 * dx;
      }
    }
  } else {
    e = -dy;
    for ( i = 0 ; i <= dy ; i++ ) {
      GR_PSET_GS(xx0,yy0, col );
      yy0 += sy;
      e += 2 * dx;
      if ( e >= 0 ) {
        xx0 += sx;
        e -= 2 * dy;
      }
    }
  }
}